from flask import Blueprint, jsonify, request
from models.base import Session
from models.author import Author
import logging
from ..middleware.auth import auth_required

logger = logging.getLogger(__name__)

bp = Blueprint('author_api', __name__)

@bp.route("/api/dashboard/authors", methods=['GET'])
@auth_required(roles=['admin', 'user'])
def get_authors():
    try:
        page = int(request.args.get('page', 1))
        per_page = int(request.args.get('per_page', 20))
        search_query = request.args.get('q')
        include_messages = request.args.get('include_messages', 'false').lower() == 'true'

        logger.info(f"Getting authors with page={page}, per_page={per_page}, search={search_query}, include_messages={include_messages}")

        try:
            if search_query:
                authors, total = Author.search(search_query, page, per_page)
            else:
                authors, total = Author.get_all(page, per_page)

            # Convert authors to dictionaries with proper error handling
            # Make sure to do this while the session is still open
            author_dicts = []
            for author in authors:
                try:
                    # Use a safer approach to convert to dict
                    author_dict = {
                        'id': author.id,
                        'name': author.name or 'Unknown',
                        'number': author.number or '',
                        'message_count': getattr(author, 'message_count', 0),
                        'created_at': author.created_at.isoformat() if author.created_at else None,
                        'last_updated': author.last_updated.isoformat() if author.last_updated else None
                    }

                    # Only try to access messages if specifically requested
                    if include_messages and hasattr(author, 'messages'):
                        try:
                            # Safely try to get message data
                            messages_data = []
                            for msg in author.messages[:10]:
                                try:
                                    messages_data.append({
                                        'id': msg.id,
                                        'caption': getattr(msg, 'caption', None),
                                        'timestamp': msg.timestamp.isoformat() if hasattr(msg, 'timestamp') and msg.timestamp else None
                                    })
                                except Exception as msg_err:
                                    logger.warning(f"Error processing message: {str(msg_err)}")

                            author_dict['messages'] = messages_data
                        except Exception as msg_list_err:
                            logger.warning(f"Error accessing messages: {str(msg_list_err)}")
                            author_dict['messages'] = []

                    author_dicts.append(author_dict)
                except Exception as author_error:
                    logger.error(f"Error converting author {author.id} to dict: {str(author_error)}")
                    # Add a minimal representation to avoid breaking the entire response
                    author_dicts.append({
                        'id': author.id,
                        'name': getattr(author, 'name', 'Unknown'),
                        'number': getattr(author, 'number', ''),
                        'error': 'Failed to load complete author data'
                    })

            return jsonify({
                "status": "success",
                "data": author_dicts,
                "pagination": {
                    "page": page,
                    "per_page": per_page,
                    "total": total,
                    "total_pages": (total + per_page - 1) // per_page
                }
            })
        except Exception as db_error:
            logger.error(f"Database error getting authors: {str(db_error)}")
            # Return an empty list instead of failing completely
            return jsonify({
                "status": "partial_success",
                "message": f"Error retrieving authors: {str(db_error)}",
                "data": [],
                "pagination": {
                    "page": page,
                    "per_page": per_page,
                    "total": 0,
                    "total_pages": 0
                }
            })
    except Exception as e:
        logger.error(f"Error in get_authors route: {str(e)}")
        return jsonify({
            "status": "error",
            "message": str(e),
            "data": [],
            "pagination": {
                "page": 1,
                "per_page": 20,
                "total": 0,
                "total_pages": 0
            }
        }), 500

@bp.route("/api/dashboard/authors/<author_id>", methods=['GET'])
@auth_required(roles=['admin', 'user'])
def get_author(author_id):
    try:
        session = Session()

        # Try to convert to int for numeric IDs
        try:
            author_id_int = int(author_id)
            author = session.query(Author).get(author_id_int)
        except (ValueError, TypeError):
            # If not a numeric ID, try string-based lookup
            author = None

        if not author:
            logger.warning(f"Author not found with ID: {author_id}")
            return jsonify({
                "status": "error",
                "message": "Author not found"
            }), 404

        # Check if we should include message details
        include_messages = request.args.get('include_messages', 'false').lower() == 'true'

        logger.info(f"Returning author {author_id} with include_messages={include_messages}")

        try:
            # Convert to dict while session is still open
            author_dict = author.to_dict(include_messages=include_messages)

            return jsonify({
                "status": "success",
                "data": author_dict
            })
        except Exception as dict_error:
            logger.error(f"Error converting author to dict: {str(dict_error)}")
            # Fallback to a minimal representation
            return jsonify({
                "status": "partial_success",
                "message": "Could not load complete author data",
                "data": {
                    "id": author.id,
                    "name": author.name,
                    "number": author.number
                }
            })
    except Exception as e:
        logger.error(f"Error getting author: {str(e)}")
        return jsonify({
            "status": "error",
            "message": str(e)
        }), 500
    finally:
        session.close()

@bp.route("/api/dashboard/authors", methods=['POST'])
@auth_required(roles=['admin'])
def create_author():
    try:
        data = request.json
        if not data:
            return jsonify({
                "status": "error",
                "message": "No data provided"
            }), 400

        # Validate required fields
        required_fields = ['name']  # Only name is required, number is optional
        missing_fields = [field for field in required_fields if field not in data]
        if missing_fields:
            return jsonify({
                "status": "error",
                "message": f"Missing required fields: {', '.join(missing_fields)}"
            }), 400

        # Let the model handle the timestamps
        # The created_at and last_updated will be set automatically
        logger.info(f"Creating new author with data: {data}")
        author = Author.create(data)
        return jsonify({
            "status": "success",
            "data": author.to_dict()
        }), 201
    except Exception as e:
        logger.error(f"Error creating author: {str(e)}")
        return jsonify({
            "status": "error",
            "message": str(e)
        }), 500

@bp.route("/api/dashboard/authors/<author_id>", methods=['PUT'])
@auth_required(roles=['admin'])
def update_author(author_id):
    try:
        data = request.json
        logger.info(f"Received update request for author {author_id} with data: {data}")

        if not data:
            return jsonify({
                "status": "error",
                "message": "No data provided"
            }), 400

        required_fields = ['name']  # Only name is required, number is optional
        missing_fields = [field for field in required_fields if field not in data]
        if missing_fields:
            return jsonify({
                "status": "error",
                "message": f"Missing required fields: {', '.join(missing_fields)}"
            }), 400

        # The last_updated field will be automatically updated by the model
        # We don't need to set it explicitly here
        logger.info(f"Processing update for author {author_id} with data: {data}")

        try:
            author = Author.update(author_id, data)
            if not author:
                logger.warning(f"Author not found for update: {author_id}")
                return jsonify({
                    "status": "error",
                    "message": "Author not found"
                }), 404

            logger.info(f"Successfully updated author {author_id}")
            return jsonify({
                "status": "success",
                "data": author.to_dict()
            })
        except Exception as update_error:
            logger.error(f"Database error updating author: {str(update_error)}")
            return jsonify({
                "status": "error",
                "message": f"Database error: {str(update_error)}"
            }), 500
    except Exception as e:
        logger.error(f"Error in update_author route: {str(e)}")
        return jsonify({
            "status": "error",
            "message": f"Server error: {str(e)}"
        }), 500

@bp.route("/api/dashboard/authors/<author_id>", methods=['DELETE'])
@auth_required(roles=['admin'])
def delete_author(author_id):
    try:
        logger.info(f"Attempting to delete author with ID: {author_id}")

        # Try to convert to int for numeric IDs
        try:
            author_id_int = int(author_id)
            # First check if the author exists
            session = Session()
            author = session.query(Author).get(author_id_int)
            session.close()

            if not author:
                logger.warning(f"Author not found for deletion: {author_id}")
                return jsonify({
                    "status": "error",
                    "message": "Author not found"
                }), 404

            # Author exists, proceed with deletion
            result = Author.delete(author_id_int)
            if result:
                logger.info(f"Successfully deleted author: {author_id}")
                return jsonify({
                    "status": "success",
                    "message": "Author deleted successfully"
                })
            else:
                logger.warning(f"Delete operation failed for author: {author_id}")
                return jsonify({
                    "status": "error",
                    "message": "Failed to delete author"
                }), 500
        except (ValueError, TypeError):
            # If not a numeric ID, return an error
            logger.error(f"Invalid author ID format: {author_id}")
            return jsonify({
                "status": "error",
                "message": "Invalid author ID format"
            }), 400
    except Exception as e:
        logger.error(f"Error deleting author: {str(e)}")
        return jsonify({
            "status": "error",
            "message": str(e)
        }), 500
